(function ($) {
  
  /**
   * Timetracking functionality.
   */
  Drupal.behaviors.timetracking = {
    attach: function (context, settings) {
      
      /**
       * Timer class.
       */
      Timer = function(timetrackingId, button){
        this.timerId;
        this.timetrackingId = timetrackingId;
        this.button = button;
        
        // Time fields.
        this.hours = 0;
        this.minutes = 0;
        this.seconds = 0;
        this.setDefaultTime();
      };
      
      /**
       * Timer class methods.
       */
      Timer.prototype = {
        
        // Start timer.
        start: function(){
          if(!this.timerId)
            this.timerId = setInterval(this.incrementTime.bind(this), 1000);
        },
        
        // Stop timer.
        stop: function(){
          if(this.timerId){
            clearTimeout(this.timerId);
            delete this.timerId;
          }
        },
        
        // Increment time on one second.
        incrementTime: function(){
          this.seconds++;
          
          if(this.seconds == 60){
            this.minutes++;
            this.seconds = 0;
          }
          if(this.minutes == 60){
            this.hours++;
            this.minutes = 0;
          }
          this.updateTime();
        },
        
        // Update timer. Renders current time in h:mm:ss format.
        updateTime: function(){
          var time = [], secText, minText, timeText;
          
          // Digitals in '99' format.
          secText = this.formatDigital(this.seconds);
          minText = this.formatDigital(this.minutes);
          
          // Collect time into array.
          if(this.hours > 0) {
            time.push(this.hours);
          }
          time.push(minText);
          time.push(secText);
          
          // Format time in h:mm:ss format.
          timeText = this.implode(':', time);
          
          // Print time.
          var id = this.timetrackingId;
          $('.timetracking_button_' + id).find('.timetracking-time-suffix').text('');
          $('.timetracking_button_' + id).find('.timetracking-time').text(timeText);
        },
        
        // Convert default time in format 9.99 in h:mm:ss and set default values
        convertTime: function(timeText){
          var minutesDecimal, timeDecimal, secondsDecimal, time = {
            hours:0,
            minutes:0,
            seconds:0
          };
          
          // Trim time string.
          timeDecimal = timeText.replace(/\s*/, '');
          
          // Check time format. It must be in digital '9.99' format.
          if(timeText.indexOf('.') != -1){
            
            // Get hours amount.
            time.hours = timeDecimal.split('.').slice(0)[0];
            
            // Calculate minutes.
            // Get decimal amount of minutes.
            minutesDecimal = 60 * (timeDecimal - time.hours);
            // Conver decimal amount of minutes to string.
            minutesDecimal = minutesDecimal + '';
            time.minutes =  minutesDecimal.split('.').slice(0)[0];
            
            // Calculate seconds.
            if(minutesDecimal.indexOf('.') != -1){
              secondsDecimal = 60 * (minutesDecimal - time.minutes);
              time.seconds = Math.round(secondsDecimal);
            }
          }
          return time;
        },
        setDefaultTime: function(){
          var timeText = this.button.find('.timetracking-time').text();
          var time = this.convertTime(timeText);
          this.hours = time.hours;
          this.minutes = time.minutes;
          this.seconds = time.seconds;
          this.updateTime();
        },
        
        // Digitals in '99' format.
        formatDigital: function(number){
          
          var numberText;
          if(number < 10) {
            numberText = '0' + number;
          }
          else {
            numberText = number;
          }
          
          return numberText;
        },
        
        // Join array elements with a string.
        implode: function( glue, pieces ){	
          return ( ( pieces instanceof Array ) ? pieces.join ( glue ) : pieces );
        }
      };
      
      var timers = [];
      
      // Autostart of enabled timer after page load.
      $('.timetracking_button').once('processed').each(function(index, item){
        
        // Get id timer id.
        var timerClass = $(this).attr('class').split(' ')[1];
        var id = timerClass.split('timetracking_button_').join(''); 
        
        if(!timers[id]){
          timer = new Timer(id, $(this));
          timers[id] = timer;
          
          // Start timer if it was enabled.
          currentState = $(this).attr("rel");
          if(currentState == 'on'){
            timer.start();
          }
        }
      });
      
      // Start / Stop on timer button click.
      $('.timetracking_button', context).click(function () {
        
        // Don't react if timer is in ajax processing.
        if($(this).hasClass('in-progress')) {
          return false;
        }
        
        // Add class for ajax processing.
        $('.timetracking_button').addClass('in-progress');
        
        // Get id of timer.
        var timerClass = $(this).attr('class').split(' ')[1];
        var id = timerClass.split('timetracking_button_').join('');
        
        // Initialize timer if undefined.
        if(!timers[id]){
          timer = new Timer(id, $(this));
          timers[id] = timer;
        }
        
        // Toggle time tracking.
        timetrackingToggle(id, timers, $(this));
        
        return false;
      });
    }
  };
  
  /**
   * Toggle time tracking.
   * 
   * @param int id
   *   Id of timer.
   * @param array timers
   *   Timers collection.
   * @param object button
   *   Clecked button DOM element.
   */
  timetrackingToggle = function(id, timers, button) {
    var timer = timers[id];
    var currentState;
    
    // Get the current state of timer from link 'rel' attribute.
    currentState = button.attr("rel");
    
    // Prepare variables for ajax call.
    if (currentState == 'on')
    {
      var toggleURL = Drupal.settings.timetracking.on.togglepath+id+'/off/ajax';
    } else 
    if (currentState == 'off')
    {
      var toggleURL = Drupal.settings.timetracking.on.togglepath+id+'/on/ajax';
    }
    
    // Change all buttons of the class. We cannot use IDs because one button 
    // could appear multiple times at one page.
    $.get(toggleURL, function(data){	
      
      // Prepare variables for new timer state.
      if (currentState == 'on')
      {
        timer.stop();
        var newState = 'off';
        var newText = Drupal.settings.timetracking.off.linktext;
        var newImage = Drupal.settings.timetracking.off.imagepath;
      } else 
      if (currentState == 'off')
      {
        timer.start();
        var newState = 'on';
        var newText = Drupal.settings.timetracking.on.linktext;
        var newImage = Drupal.settings.timetracking.on.imagepath;
      }
      
      // First stop all timetrackings. 
      // Doesn't matter if we start or stop timer, other timers musn't run.
      // Get all active timers.
      $('.timetracking_button[rel="on"]').each(function(index, button_element){
        // Set 'off' state.
        $(this).attr("rel", 'off');
        var timerClass = $(this).attr('class').split(' ')[1];
        var otherTimerId = timerClass.split('timetracking_button_').join('');
        
        // Stop other timers.
        if(otherTimerId != id && timers[otherTimerId]){
          timers[otherTimerId].stop();
          // Set 'off' image.
          offImage = Drupal.settings.timetracking.off.imagepath;
          $(this).find('.timetracking_button_image').attr("src", offImage);
          // Set 'off' text.
          offText = Drupal.settings.timetracking.off.linktext;
          $(this).find('.timetracking_text').text(offText);
        }
      });
      
      // Start triggered timer.
      // Set new state.
      $('.timetracking_button_' + id).attr("rel", newState);
      // Set new image.
      $('.timetracking_button_' + id).find('.timetracking_button_image').attr("src", newImage);
      // Set new text.
      $('.timetracking_button_' + id).find('.timetracking_text').text(newText);
      
      // Remove class for ajax processing. Processing is finished.
      $('.timetracking_button').removeClass('in-progress');
    });
  };

})(jQuery);
